<?php
#BEGIN_LICENSE
#-------------------------------------------------------------------------
# Module: Newsletter Made Simple (c) 2012 by Robert Campbell 
#         (calguy1000@cmsmadesimple.org)
#  An addon module for CMS Made Simple to provide a flexible
#  mailing list solution.
# 
#-------------------------------------------------------------------------
# CMS - CMS Made Simple is (c) 2005 by Ted Kulp (wishy@cmsmadesimple.org)
# This project's homepage is: http://www.cmsmadesimple.org
#
#-------------------------------------------------------------------------
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# However, as a special exception to the GPL, this software is distributed
# as an addon module to CMS Made Simple.  You may not use this software
# in any Non GPL version of CMS Made simple, or in any version of CMS
# Made simple that does not indicate clearly and obviously in its admin 
# section that the site was built with CMS Made simple.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
# Or read it online: http://www.gnu.org/licenses/licenses.html#GPL
#
#-------------------------------------------------------------------------
#END_LICENSE

final class nms_process_job 
{
  private function __construct() {}

  private static function insert_job_email($jobid, $messageid, $email, $uniqueid, $username)
  {
    $db = cmsms()->GetDb();

    $iquery = 'INSERT INTO '.NMS_MAILS_TABLE.' (jobid, messageid, email, uniqueid, username) 
               VALUES (?,?,?,?,?)';
    $dbr = $db->Execute($iquery,array($jobid,$messageid,$email,$uniqueid,$username));
    if( !$dbr ) return FALSE;
    return TRUE;
  }

  public static function setup_mails(cgjobmgr_task& $task)
  {
    // get the job id...
    $nms = cms_utils::get_module('NMS');
    $nms_jobid = $task->get_persistent_data('nms_jobid');
    if( !$nms_jobid ) {
      audit('',$nms->GetName(),'nms_jobid not found');
      return cgjobmgr_task::STATUS_ERROR;
    }

    // get the job, and parts.
    $db = cmsms()->GetDb();
    $query = 'SELECT * FROM '.NMS_JOBS_TABLE.' WHERE jobid = ?';
    $nms_job = $db->GetRow($query,array($nms_jobid));
    if( !is_array($nms_job) ) {
      $query = 'UPDATE '.NMS_JOBS_TABLE.' SET status = ?, started = NOW() WHERE jobid = ?';
      $dbr = $db->Execute($query,array(NMS_JOBSTATUS_ERROR,$nms_jobid));
      audit('',$nms->GetName(),'nms job not found');
      return cgjobmgr_task::STATUS_ERROR;
    }
    $query = 'SELECT * FROM '.NMS_JOB_PARTS_TABLE.' WHERE jobid = ?';
    $parts = $db->GetArray($query,array($nms_jobid));
    if( !is_array($parts) || count($parts) == 0 ) {
      $query = 'UPDATE '.NMS_JOBS_TABLE.' SET status = ?, started = NOW() WHERE jobid = ?';
      $dbr = $db->Execute($query,array(NMS_JOBSTATUS_ERROR,$nms_jobid));
      audit('',$nms->GetName(),'nms job parts not found');
      return cgjobmgr_task::STATUS_ERROR;
    }
    if( $nms_job['status'] != NMS_JOBSTATUS_STARTING ) {
      $query = 'UPDATE '.NMS_JOBS_TABLE.' SET status = ?, started = NOW() WHERE jobid = ?';
      $dbr = $db->Execute($query,array(NMS_JOBSTATUS_ERROR,$nms_jobid));
      audit('',$nms->GetName(),'nms job has invalid status ');
      return cgjobmgr_task::STATUS_ERROR;
    }

    // get the lists
    $query = 'SELECT * FROM '.NMS_LIST_TABLE;
    $lists = $db->GetArray($query);
    if( !is_array($lists) ) {
      $query = 'UPDATE '.NMS_JOBS_TABLE.' SET status = ?, started = NOW() WHERE jobid = ?';
      $dbr = $db->Execute($query,array(NMS_JOBSTATUS_ERROR,$nms_jobid));
      audit('',$nms->GetName(),'nms lists not found');
      return cgjobmgr_task::STATUS_ERROR;
    }
    $lists = cge_array::to_hash($lists,'listid');

    // loop through the parts
    foreach( $parts as $one_part ) {

      // get the list for the part
      if( !isset($lists[$one_part['listid']]) ) {
	audit('',$nms->GetName(),'nms list '.$one_part['listid'].' not found');
	return cgjobmgr_task::STATUS_ERROR;
      }

      if( $lists[$one_part['listid']]['dynamic'] ) {
	$list = $lists[$one_part['listid']];
	// it's a dynamic list
	$parms = array();
	$res = UserTagOperations::get_instance()->CallUserTag($list['dynamic'],$parms);

	if( is_array($res) && count($res) ) {
	  $n = 0;
	  foreach( $res as $one_result ) {
	    if( is_string($one_result) ) {
	      $email = $one_result;
	    }
	    else if( is_array($one_result) ) {
	      if( !isset($one_result['email']) ) continue;
	      $username = '';
	      $email = trim($one_result['email']);
	      if( isset($one_result['username']) ) {
	        $username = trim($one_result['username']);
	      }
	    }
	    if( !$email ) continue;
	    if( !is_email($email) ) continue; // a little pre-emptive striking.
	    $x = self::insert_job_email($one_part['jobid'],$one_part['messageid'],
					$email,'',$username);
	    if( $x ) $n++;
	  }
	  audit('',$nms->GetName(),'Added '.$n.' eails from udt '.$list['dynamic'].' to job '.$nms_jobid);
	}
      }
      else {
	// its a static list....
	$q = "SELECT DISTINCT jobid,email,username,C.uniqueid,messageid FROM ".
              NMS_LISTUSER_TABLE." A left join ".
              NMS_JOB_PARTS_TABLE." B on A.listid = B.listid left join ".
              NMS_USERS_TABLE." C on A.userid = C.userid 
	      WHERE C.disabled = 0 AND C.confirmed = 1 
                AND COALESCE(C.error_count,0) < ? 
                AND B.jobid = ? AND B.listid = ? AND B.messageid = ?";
	$dbr = $db->Execute( $q, 
			     array( $nms->GetPreference('max_error_count',5), 
				    (int)$nms_jobid, 
				    (int)$one_part['listid'], 
				    (int)$one_part['messageid'] ));

	if( !$dbr || $dbr->RecordCount() == 0 ) {
	  $query = 'UPDATE '.NMS_JOBS_TABLE.' SET status = ?, started = NOW() WHERE jobid = ?';
	  $dbr = $db->Execute($query,array(NMS_JOBSTATUS_ERROR,$nms_jobid));
	  audit('',$nms->GetName(),'no emails found ');
	  return cgjobmgr_task::STATUS_ERROR;
	}

	$n = 0;
	while( $dbr && !$dbr->EOF ) {
	  $row = $dbr->fields;
	  $x = self::insert_job_email($row['jobid'],$row['messageid'],
				      $row['email'],$row['uniqueid'],
				      $row['$username']);
	  if( $x ) $n++;
	  $dbr->MoveNext();
	}
      }
    } // for each part
          
    // set the nms job status to RUNNING
    $query = 'UPDATE '.NMS_JOBS_TABLE.' SET status = ?, started = NOW() WHERE jobid = ?';
    $dbr = $db->Execute($query,array(NMS_JOBSTATUS_RUNNING,$nms_jobid));
    
    return cgjobmgr_task::STATUS_COMPLETE;
  }

  public static function send_emails(cgjobmgr_task& $task)
  {
    static $test_jobid = -1;
    static $message = null;

    // how many messages can we send?
    $interval = max(1,$task->get_run_interval());
    $nms = cms_utils::get_module('NMS');
    $batch_size = 25;
    $messagesperhour = (int)$nms->GetPreference('messagesperhour',0);
    if( $messagesperhour > 0 ) {
      $messagespermin = (int)($messagesperhour / 60);
      $batch_size = $messagespermin * $interval;
    }

    $nms_jobid = $task->get_persistent_data('nms_jobid');
    if( !$nms_jobid ) {
      return cgjobmgr_task::STATUS_ERROR;
    }
    if( $test_jobid < 0 ) {
      $test_jobid = $nms_jobid;
      
      $db = cmsms()->GetDb();
      $query = 'SELECT * FROM '.NMS_JOBS_TABLE.' WHERE jobid = ?';
      $nms_job = $db->GetRow($query,array($nms_jobid));
      if( !is_array($nms_job) ) {
	return cgjobmgr_task::STATUS_ERROR;
      }
      if( $nms_job['status'] != NMS_JOBSTATUS_RUNNING ) {
	return cgjobmgr_task::STATUS_ERROR;
      }
    }

    // 1.  get N records for sending
    $db = cmsms()->GetDb();
    $query = 'SELECT * FROM '.NMS_MAILS_TABLE.' where jobid = ? AND status IS NULL ORDER BY messageid LIMIT ?';
    $dbr = $db->GetArray($query,array($nms_jobid,$batch_size));
    if( !is_array($dbr) || count($dbr) == 0 ) {
      // no more messages to send
      return cgjobmgr_task::STATUS_COMPLETE;
    }
    
    foreach( $dbr as $one ) {
      // get the message 
      if( is_null($message) || !is_array($message) || !isset($message['messageid']) || $message['messageid'] != $one['messageid'] ) {
	$query = 'SELECT * FROM '.NMS_MESSAGES_TABLE.' WHERE messageid = ?';
	$message = $db->GetRow($query,array($one['messageid']));
      }
      
      // send
      $res = $nms->_ProcessOneEmail('cntnt01',$message,$one,$nms_jobid);
      
      // update records.
      $status = 'sent';
      if( !$res ) $status = 'error';
      $query = 'UPDATE '.NMS_MAILS_TABLE.' SET senddate = ?, status = ? WHERE jobid = ? AND messageid = ? AND email = ?';
      $dbr = $db->Execute($query,array(time(),$status,$one['jobid'],$one['messageid'],$one['email']));
    }

    return cgjobmgr_task::STATUS_INCOMPLETE;
  }

  public static function finish(cgjobmgr_task& $task)
  {
    $nms_jobid = $task->get_persistent_data('nms_jobid');
    if( !$nms_jobid ) {
      return cgjobmgr_task::STATUS_ERROR;
    }

    // set NMS job to finished
    $db = cmsms()->GetDb();

    $query = 'UPDATE '.NMS_JOBS_TABLE.' SET status = ?, finished = NOW() WHERE jobid = ?';
    $db->Execute($query,array(NMS_JOBSTATUS_FINISHED,$nms_jobid));

    $query = 'SELECT * FROM '.NMS_JOBS_TABLE.' WHERE jobid = ?';
    $nms_job = $db->GetRow($query,array($nms_jobid));

    // check if we're sending mails
    $nms = cms_utils::get_module('NMS');
    $tmp = $nms->GetPreference('send_jobcomplete_groups');
    if( $tmp > 0 ) {
      $emaillist = array();
      $ops = cmsms()->GetUserOperations();
      $list = $ops->LoadUsersInGroup($tmp);
      if( is_array($list) && count($list) ) {
	foreach( $list as $user ) {
	  if( $user->email != '' && !in_array($user->email,$emaillist) ) {
	    $emaillist[] = $user->email;
	  }
	}
      }

      if( count($emaillist) ) {
	$smarty = cmsms()->GetSmarty();

	// gather statistics
	// - total rows
	$query = 'SELECT count(messageid) FROM '.NMS_MAILS_TABLE.' WHERE jobid = ?';
	$nrows = $db->GetOne($query,array($nms_jobid));
	$smarty->assign('numrows',$nrows);

        // - total errors
	$query = 'SELECT email,username FROM '.NMS_MAILS_TABLE.' WHERE jobid = ? AND status =  ?';
	$nerrors = 0;
	$errors = $db->GetArray($query,array($nms_jobid,'error'));
	if( is_array($errors) && count($errors) ) {
	  $nerrors = count($errors);
	  $smarty->assign('errors',$errors);
	}
	$smarty->assign('nerrors',$nerrors);

	$stime = $db->UnixTimeStamp($nms_job['started']);
	$smarty->assign('stime',$stime);
	$etime = $db->UnixTimeStamp($nms_job['finished']);
	$smarty->assign('etime',$etime);
	$dtime = $etime - $stime;
	$smarty->assign('dtime',$dtime);

	$smarty->assign('job',$nms_job);

	// get the parts for the job.' 
	$query = 'SELECT DISTINCT M.subject FROM '.NMS_MESSAGES_TABLE.' M LEFT JOIN '.NMS_JOB_PARTS_TABLE.' P ON M.messageid = P.messageid WHERE P.jobid = ?';
	$subject = $db->GetOne($query,array($nms_jobid));
	$smarty->assign('subject',$subject);

	$query = 'SELECT DISTINCT L.name FROM '.NMS_LIST_TABLE.' L LEFT JOIN '.NMS_JOB_PARTS_TABLE.' P ON L.listid = P.listid WHERE P.jobid = ?';
	$lists = $db->GetCol($query,array($nms_jobid));
	$smarty->assign('lists',$lists);

	// send mails
	$mailer = cms_utils::get_module('CMSMailer');
	$mailer->reset();
	$mailer->SetCharSet($nms->GetPreference('message_charset'));
	$mailer->SetFromName($nms->GetPreference('admin_name'));
	$mailer->SetFrom($nms->GetPreference('admin_email'));
	$replyto = $nms->GetPreference('admin_replyto');
	if( !empty($replyto) ) {
	  $mailer->AddReplyTo($replyto);
	}
	$mailer->AddCustomHeader("X-Mailer: CMS Made Simple ".CMS_VERSION.' ('.$nms->GetName()." ".$nms->GetVersion().' )');
	$mailer->SetSubject($nms->GetPreference('send_jobcomplete_subject'));
	$mailer->IsHTML(true);
	foreach( $emaillist as $one ) {
	  $mailer->AddAddress($one);
	}
	$body = $nms->ProcessTemplateFromDatabase('send_jobcomplete_template');
	$mailer->SetBody($body);
	$mailer->Send();
      }
    }
    // cleanup mails table

    audit($nms_jobid,$nms->GetName(),'Job Finished: '.$nms_job['title']);
    return cgjobmgr_task::STATUS_COMPLETE;
  }
} // end of class

#
# EOF
#
?>
